package java.security.cert;

import java.io.ByteArrayInputStream;
import java.io.ObjectStreamException;
import java.io.NotSerializableException;
import java.util.List;
import java.util.Iterator;
import java.io.Serializable;

public abstract class CertPath implements Serializable
{
    private static final long serialVersionUID = 6068470306649138683L;
    private String type;
    
    protected CertPath(String type) {
        this.type = type;
    }
    
    public String getType() {
        return this.type;
    }
    
    public abstract Iterator<String> getEncodings();
    
    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof CertPath)) {
            return false;
        }
        CertPath certPath = (CertPath)o;
        return certPath.getType().equals(this.type) && this.getCertificates().equals(certPath.getCertificates());
    }
    
    @Override
    public int hashCode() {
        return 31 * this.type.hashCode() + this.getCertificates().hashCode();
    }
    
    @Override
    public String toString() {
        StringBuffer sb = new StringBuffer();
        Iterator<Object> iterator = this.getCertificates().iterator();
        sb.append("\n" + this.type + " Cert Path: length = " + this.getCertificates().size() + ".\n");
        sb.append("[\n");
        for (int n = 1; iterator.hasNext(); ++n) {
            sb.append("=========================================================Certificate " + n + " start.\n");
            sb.append(((Certificate)iterator.next()).toString());
            sb.append("\n=========================================================Certificate " + n + " end.\n\n\n");
        }
        sb.append("\n]");
        return sb.toString();
    }
    
    public abstract byte[] getEncoded() throws CertificateEncodingException;
    
    public abstract byte[] getEncoded(String p0) throws CertificateEncodingException;
    
    public abstract List<? extends Certificate> getCertificates();
    
    protected Object writeReplace() throws ObjectStreamException {
        try {
            return new CertPath.CertPathRep(this.type, this.getEncoded());
        }
        catch (CertificateException cause) {
            NotSerializableException ex = new NotSerializableException("java.security.cert.CertPath: " + this.type);
            ex.initCause(cause);
            throw ex;
        }
    }
    
    protected static class CertPathRep implements Serializable
    {
        private static final long serialVersionUID = 3015633072427920915L;
        private String type;
        private byte[] data;
        
        protected CertPathRep(String type, byte[] data) {
            this.type = type;
            this.data = data;
        }
        
        protected Object readResolve() throws ObjectStreamException {
            try {
                return CertificateFactory.getInstance(this.type).generateCertPath(new ByteArrayInputStream(this.data));
            }
            catch (CertificateException cause) {
                NotSerializableException ex = new NotSerializableException("java.security.cert.CertPath: " + this.type);
                ex.initCause(cause);
                throw ex;
            }
        }
    }
}
